//
//  GetFileView.swift
//  SampleApp
//
//  Created by Zay Yar Htun on 7/7/23.
//  Copyright © 2023 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct GetFileView: View {
    @State private var fileId: String = ""
    @StateObject var viewModel = GetFileViewModel()
    
    var body: some View {
        Form {
            TextField("Enter File Id", text: $fileId)
            
            Section {
                Button {
                    guard !fileId.isEmpty else { return }
                    
                    viewModel.getFile(id: fileId)
                    
                } label: {
                    HStack {
                        Spacer()
                        Text("Get File")
                        Spacer()
                    }
                }
            }
            
            Section(header: Text("Here you will see result received from sdk")) {
                VStack(alignment: .leading) {
                    Text("Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                        .padding(.bottom, 12)
                    Text(viewModel.result)
                }
            }
        }
        .navigationTitle("Get File")
    }
}

struct GetFileView_Previews: PreviewProvider {
    static var previews: some View {
        GetFileView()
    }
}

class GetFileViewModel: ObservableObject {
    
    @Published var result: String = ""
    var fileURL: String = ""
    var fileId: String = ""
    var fileType: String = ""
    
    var token: AmityNotificationToken?
    var fileRepo = AmityFileRepository(client: AmityManager.shared.client!)
    
    func getFile(id: String) {
        Task { @MainActor in
            do {
                let rawFile = try await fileRepo.getFile(fileId: id)
                fileId = rawFile.fileId
                fileType = rawFile.type.rawValue
                switch rawFile.type {
                case .file:
                    fileURL = rawFile.mapToFileData()?.fileURL ?? ""
                case .image:
                    fileURL = rawFile.mapToImageData()?.fileURL ?? ""
                case .video:
                    fileURL = rawFile.mapToVideoData()?.fileURL ?? ""
                @unknown default:
                    fileURL =  ""
                }
                result = "FileId:\n\(fileId)\n\nFileType:\n \(fileType)\n\nFileURL:\n\(fileURL)"
            } catch {
                result = error.localizedDescription
                print("Error when getting rawFile: \(error)")
            }
        }
    }
}
