//
//  SDKModel+Extension.swift
//  SampleApp
//
//  Created by Nishan Niraula on 11/24/21.
//  Copyright © 2021 David Zhang. All rights reserved.
//

import Foundation
import AmitySDK

extension AmityPost {
    
    var modelDescription: String {
        return """
            Post:
            
            Id: \(self.postId)
            Data Type: \(self.dataType)
            Data: \(String(describing: self.data))
            Child Posts Count: \(self.childrenPosts.count ?? 0)
            Reactions Count: \(self.reactionsCount)
            Reactions: \(String(describing: self.reactions))
            Flag Count: \(self.flagCount)
            Comments Count: \(self.commentsCount)
            Target Community: \(self.targetCommunity?.communityId ?? "")
            Deleted Status: \(self.isDeleted)
            """
    }
}

extension AmityUser {
    
    var modelDescription: String {
        
        let userRoles = self.roles as? [String] ?? []
        
        return """
            User:
            
            Id: \(self.userId)
            Display Name: \(self.displayName ?? "-")
            User Description: \(self.userDescription)
            Roles: \(userRoles.joined(separator: ","))
            Flag Count: \(self.flagCount)
            Avatar FileId: \(self.avatarFileId ?? "")
            Avatar Custom URL: \(self.avatarCustomUrl ?? "")
            """
    }
}

extension AmityCommunity {
    
    var modelDescription: String {
        return """
            Community:
            
            Id: \(self.communityId)
            Display Name: \(self.displayName)
            Description: \(self.communityDescription)
            Channel Id: \(self.channelId)
            Post Count: \(self.postsCount)
            Members Count: \(self.membersCount)
            Public Status: \(self.isPublic)
            Joined Status: \(self.isJoined)
            Deleted Status: \(self.isDeleted)
                        
            Post Settings: \(self.postSettings.title)
            Only Admin Can Post: \(self.onlyAdminCanPost)
            Post Review Enabled: \(self.isPostReviewEnabled)
            Avatar File Id: \(self.avatar?.fileId ?? "")
            Avatar File URL: \(self.avatar?.fileURL ?? "")
            """
    }
}

extension AmityComment {
    
    var modelDescription: String {
        return """
            Comment:
            
            Id: \(self.commentId)
            Data: \(String(describing: self.data))
            ReferenceId: \(self.referenceId)
            ReferenceType: \(self.referenceType.description)
            UserId: \(self.userId)
            Data Type: \(self.dataType.description)
            Reactions: \(String(describing: self.reactions))
            My Reactions: \(self.myReactions)
            Flag Count: \(self.flagCount)
            Replies Count: \(self.childrenNumber)
            Deleted Status: \(self.isDeleted)
            Sync State: \(self.syncState.description)
            """
    }
}

extension AmityCommentReferenceType {
    
    var description: String {
        switch self {
        case .content:
            return "Content"
        case .post:
            return "Post"
        @unknown default:
            return ""
        }
    }
}

extension AmityDataType {
    
    var description: String {
        switch self {
        case .text:
            return "Text"
        default:
            return ""
        }
    }
}

extension AmityChannel {
    
    var modelDescription: String {
        return """
        Channel:
        
        Id: \(self.channelId)
        Type: \(self.channelType.description)
        Metadata: \(self.metadata?.description ?? "")
        Tags: \(self.tags)
        Deleted: \(self.isDeleted)
        Display Name: \(self.displayName ?? "")
        Muted: \(self.isMuted)
        Channel Unread Count: \(self.unreadCount)
        Has Mentioned: \(self.hasMentioned)
        Member Count: \(self.memberCount)
        Moderator Member Count: \(self.moderatorCount)
        Avatar Info: \(self.getAvatarInfo()?.fileId ?? "")
        """
    }
}

extension AmityMessage {
    
    var modelDescription: String {
        return """
        Message:
        
        Message Id: \(self.messageId)
        LocalMessage Id: \(self.uniqueId)
        User Id: \(self.userId)
        Channel Id: \(self.channelId)
        Subchannel Id: \(self.subChannelId)
        Type: \(self.messageType.description)
        Message Data: \(self.data?.description ?? "")
        Metadata: \(self.metadata?.description ?? "")
        Tags: \(self.tags)
        Deleted: \(self.isDeleted)
        Parent Id: \(self.parentId ?? "")
        Children Count: \(self.childrenNumber)
        Reaction: \(self.reactions ?? [:])
        Create At: \(self.createdAt)
        Edited At: \(self.editedAt)
        Is Edited: \(self.isEdited)
        Is Deleted: \(self.isDeleted)
        Flag count: \(self.flagCount)
        Read count: \(self.readCount)
        Delivered count: \(self.deliveredCount)
        """
    }
}

extension AmitySubChannel {
    
    var modelDescription: String {
        return """
        SubChannel:
        
        Sub Channel Name: \(displayName)
        Channel Id: \(channelId)
        SubChannel Id: \(subChannelId)
        Type: \(channelType)
        Channel Id: \(channelId)
        Message Count: \(messageCount)
        Is Deleted: \(isDeleted)
        latest Message Id: \(String(describing: latestMessageId))
        Last Activity: \(String(describing: lastActivity))
        Creator Id: \(creatorId)
        Created At: \(String(describing: createdAt)))
        Updated At: \(String(describing: updatedAt))
        Edited At: \(String(describing: editedAt))
        Unread Count: \(unreadCount)
        Is Support Unread Count: \(isUnreadCountSupported)
        """
    }
}

extension AmityCommunityQueryFilter: CaseIterable, CustomStringConvertible {
    public static var allCases: [AmityCommunityQueryFilter] = [.all, .userIsMember, .userIsNotMember]
    
    public var description: String {
        switch self {
        case .all:
            return "All"
        case .userIsMember:
            return "Member"
        case .userIsNotMember:
            return "Not Member"
        @unknown default:
            fatalError()
        }
    }
}



extension AmityCommunitySortOption: CaseIterable, CustomStringConvertible {
    public static var allCases: [AmityCommunitySortOption] = [.displayName, .lastCreated, .firstCreated]
    
    public var description: String {
        switch self {
        case .lastCreated:
            return "Last Created"
        case .firstCreated:
            return "First Created"
        case .displayName:
            return "Display Name"
        @unknown default:
            return ""
        }
    }
}


extension AmityLoadingStatus {
    
    var description: String {
        switch self {
        case .notLoading:
            return "Not Loading"
        case .loading:
            return "Loading"
        case .loaded:
            return "Loaded"
        case .error:
            return "Error"
        @unknown default:
            return ""
        }
    }
}
